#target illustrator

function main() {
    // Create a new dialog
    var dlg = new Window('dialog', 'Label Color Separations');

    // Add a panel for location selection
    var locationPanel = dlg.add('panel', undefined, 'Select Print Location');
    locationPanel.orientation = 'column';

    var frontCheck = locationPanel.add('checkbox', undefined, 'Front');
    var backCheck = locationPanel.add('checkbox', undefined, 'Back');
    var pocketCheck = locationPanel.add('checkbox', undefined, 'Pocket');
    var rightSleeveCheck = locationPanel.add('checkbox', undefined, 'Right Sleeve');
    var leftSleeveCheck = locationPanel.add('checkbox', undefined, 'Left Sleeve');
    var rightLegCheck = locationPanel.add('checkbox', undefined, 'Right Leg');
    var leftLegCheck = locationPanel.add('checkbox', undefined, 'Left Leg');

    // Get the colors used in the artwork
    var usedColors = getUsedColors();

    // Add a panel for spot colors
    var colorPanel = dlg.add('panel', undefined, 'Select Spot Colors');
    colorPanel.orientation = 'column';
    var colorChecks = [];
    for (var i = 0; i < usedColors.length; i++) {
        var colorCheck = colorPanel.add('checkbox', undefined, usedColors[i]);
        colorChecks.push(colorCheck);
    }

    // Add OK and Cancel buttons
    dlg.add('button', undefined, 'OK', {name: 'ok'});
    dlg.add('button', undefined, 'Cancel', {name: 'cancel'});

    // Add script version information at the bottom
    dlg.add('statictext', undefined, 'Label Script V4.2 © Townsend 2024');

    // Show the dialog and act based on user input
    var result = dlg.show();
    if (result == 1) { // OK button clicked
        var selectedLocations = [];
        if (frontCheck.value) selectedLocations.push('Front');
        if (backCheck.value) selectedLocations.push('Back');
        if (pocketCheck.value) selectedLocations.push('Pocket');
        if (rightSleeveCheck.value) selectedLocations.push('Right Sleeve');
        if (leftSleeveCheck.value) selectedLocations.push('Left Sleeve');
        if (rightLegCheck.value) selectedLocations.push('Right Leg');
        if (leftLegCheck.value) selectedLocations.push('Left Leg');

        var selectedColors = [];
        for (var i = 0; i < colorChecks.length; i++) {
            if (colorChecks[i].value) {
                selectedColors.push(usedColors[i]);
            }
        }

        addLabelsToArtwork(selectedLocations, selectedColors);
    }
}

function getUsedColors() {
    var doc = app.activeDocument;
    var usedColors = [];
    var colorNames = {};

    function addColor(color) {
        var colorName = color.spot ? color.spot.name : color.typename;
        if (!colorNames[colorName]) {
            colorNames[colorName] = true;
            usedColors.push(colorName);
        }
    }

    function checkPathItem(pathItem) {
        if (pathItem.filled) {
            addColor(pathItem.fillColor);
        }
        if (pathItem.stroked) {
            addColor(pathItem.strokeColor);
        }
    }

    function traverseGroupItems(groupItem) {
        for (var i = 0; i < groupItem.pageItems.length; i++) {
            var item = groupItem.pageItems[i];
            if (item.typename == "PathItem") {
                checkPathItem(item);
            } else if (item.typename == "GroupItem") {
                traverseGroupItems(item);
            }
        }
    }

    // Traverse the document to get all used colors
    for (var i = 0; i < doc.pageItems.length; i++) {
        var item = doc.pageItems[i];
        if (item.typename == "PathItem") {
            checkPathItem(item);
        } else if (item.typename == "GroupItem") {
            traverseGroupItems(item);
        }
    }

    return usedColors;
}

function addLabelsToArtwork(locations, colors) {
    var doc = app.activeDocument;
    var selection = doc.selection;
    if (selection.length === 0) {
        alert("Please select the artwork before running the script.");
        return;
    }

    // Get the bounds of the selected artwork
    var artworkBounds = selection[0].visibleBounds;
    var xOffset = artworkBounds[0];
    var yOffset = artworkBounds[1] + 90; // 1.25 inches (90 points) above the top edge of the artwork
    var maxWidth = artworkBounds[2] - artworkBounds[0]; // Maximum width for the label text

    var textFrames = []; // Array to hold text frames for conversion to outlines
    var fileName = doc.name.split('.')[0]; // Get the file name without extension

    // Create the label content
    var locationText = locations.join(" - ");
    var colorText = colors.join(" - ");
    var labelContent = fileName + " - " + locationText + " - " + colorText;

    // Split the label content into multiple lines if it exceeds the maximum width
    var lines = [];
    var words = labelContent.split(" ");
    var currentLine = "";

    var dummyTextItem = doc.textFrames.add();
    dummyTextItem.textRange.characterAttributes.size = 14; // Set font size to 14 pt
    dummyTextItem.textRange.characterAttributes.textFont = app.textFonts.getByName("Arial-BoldMT"); // Set font to bold (Arial-BoldMT)

    for (var i = 0; i < words.length; i++) {
        dummyTextItem.contents = currentLine + (currentLine ? " " : "") + words[i];
        if (dummyTextItem.width > maxWidth) {
            lines.push(currentLine);
            currentLine = words[i];
        } else {
            currentLine += (currentLine ? " " : "") + words[i];
        }
    }
    if (currentLine) {
        lines.push(currentLine);
    }
    dummyTextItem.remove();

    // Add each line of text as a separate text frame
    for (var i = 0; i < lines.length; i++) {
        var textItem = doc.textFrames.add();
        textItem.contents = lines[i];
        textItem.position = [xOffset, yOffset - (i * 20)]; // Adjust yOffset for each line
        textItem.textRange.characterAttributes.size = 14; // Set font size to 14 pt
        textItem.textRange.characterAttributes.textFont = app.textFonts.getByName("Arial-BoldMT"); // Set font to bold (Arial-BoldMT)
        textFrames.push(textItem);
    }

    // Set color of file name and location text to registration color
    var registrationColor = doc.swatches.getByName("[Registration]").color;
    var nameAndLocationLength = (fileName + " - " + locationText).length;
    var currentLength = 0;

    for (var i = 0; i < textFrames.length; i++) {
        var textItem = textFrames[i];
        var textRange = textItem.textRange.characters[0];
        textRange.length = textItem.contents.length;

        if (currentLength < nameAndLocationLength) {
            var remainingLength = nameAndLocationLength - currentLength;
            textRange.length = Math.min(remainingLength, textItem.contents.length);
            textRange.characterAttributes.fillColor = registrationColor;
        }

        currentLength += textItem.contents.length + 1; // +1 for the space character
    }

    // Set color of each word to its respective spot color
    var startIndex = nameAndLocationLength + 3; // +3 for " - "
    var colorIndex = 0;

    for (var i = 0; i < textFrames.length; i++) {
        var textItem = textFrames[i];
        var textRange = textItem.textRange;

        var rangeStart = 0;

        while (rangeStart < textRange.length && colorIndex < colors.length) {
            var colorName = colors[colorIndex];
            var wordStartIndex = textItem.contents.indexOf(colorName, rangeStart);

            if (wordStartIndex !== -1) {
                var wordRange = textRange.characters[wordStartIndex];
                wordRange.length = colorName.length;
                wordRange.characterAttributes.fillColor = doc.swatches.getByName(colorName).color;
                rangeStart = wordStartIndex + colorName.length + 3; // +3 for " - "
                colorIndex++;
            } else {
                rangeStart = textRange.length; // Move to next line
            }
        }
    }

    // Create a dashed grey line above the text in process color
    var line = doc.pathItems.add();
    line.setEntirePath([
        [xOffset, yOffset + (0.11 * 72)], // Start point
        [xOffset + maxWidth, yOffset + (0.11 * 72)] // End point
    ]);

    // Set line color to process black
    var processBlack = new CMYKColor();
    processBlack.cyan = 0;
    processBlack.magenta = 0;
    processBlack.yellow = 0;
    processBlack.black = 100;

    line.strokeColor = processBlack;
    line.strokeWidth = 1;
    line.strokeDashes = [3, 3]; // Create a dashed line

    // Convert text frames to outlines
    for (var i = 0; i < textFrames.length; i++) {
        convertToOutlines(textFrames[i]);
    }
}

function convertToOutlines(textFrame) {
    app.executeMenuCommand('deselectall');
    textFrame.selected = true;
    app.executeMenuCommand('outline');
}

main();
